#include "mastodon_api.h"
#include "http/server/request.h"

// Submodules
#include "form.h"
#include "json/json.h"
#include "http/query.h"
#include "ffdb/fs_list.h"

#include "model/status.h"
#include "model/account.h"
#include "model/notification.h"
#include "model/server.h"
#include "model/ap/activity.h"

#include "controller/pleroma_api.h"
#include "controller/api/timeline.h"
#include "controller/api/client_apps.h"
#include "controller/api/status.h"
#include "controller/api/notice.h"
#include "controller/api/accounts.h"

#include <stddef.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

bool http_request_route_id( struct http_request* req, int* id )
{
	char* id_str = http_request_route_get_dir_or_file(req);
	if( !id_str || !*id_str ) { return false; }
	*id = -1;
	sscanf( id_str, "%d", id );
	free(id_str);
	if( *id == -1 ) { return false; }

	return true;
}
// Route: /api/v1/statuses/%d{id}/
bool route_statuses_1f( struct http_request* req )
{
	bool result = false;
	struct status* s = NULL;

	if( http_request_route_term( req, "" ) && http_request_route_method( req, "POST" ) ) {
		struct account* owner = account_from_id(owner_account_id);
		bool res = handle_post(req, owner);
		account_free(owner);
		return res;
	}

	if( !http_request_route( req, "/" ) ) { return false; }

	int id = -1;

	if( !http_request_route_id( req, &id ) ) { goto failed; }
	s = status_from_id(id);
	if( !s ) { goto failed; }

	if( http_request_route( req, "context" ) ) {
		show_status_context( req, s );
		goto success;
	} else if( http_request_route_term( req, "reblog" ) ) {
		if( handle_repost( req, s ) ) { goto success; }
	} else if( http_request_route_term( req, "" ) ) {
		show_status( req, s );
		goto success;
	}
failed:
	result = false;
cleanup:
	status_free(s);
	return result;
success:
	result = true;
	goto cleanup;
}

bool route_mastodon_api( struct http_request* req )
{
	if( http_request_route_term( req, "apps" ) ) {
		if( http_request_route_method( req, "POST" ) ) {
			return handle_mastodon_api_apps(req);
		}
	} else if( http_request_route( req, "timelines/public" ) ) {
		return handle_timeline( req, tli_public );
	} else if( http_request_route( req, "instance" ) ) {
		http_request_send_headers( req, 200, "application/json", true );
		FILE* f = http_request_get_response_body( req );
		#include "src/view/api/instance_data.json.inc"
		return true;
	}

	if( !check_bearer_token(req) ) { return false; }

	if( http_request_route( req, "pleroma" ) ) {
		return route_pleroma_api( req );
	}

	if( http_request_route( req, "notifications" ) ) {
		return handle_notifications(req);
	} else if( http_request_route( req, "filters" ) ) {
		http_request_send_headers( req, 200, "application/json", true );
		FILE* f = http_request_get_response_body( req );
		fprintf( f, "[]" );
		return true;
	} else if( http_request_route( req, "statuses" ) ) {
		return route_statuses_1f(req);
	} else if( http_request_route( req, "timelines/" ) ) {
		if( http_request_route( req, "home" ) ) {
			return handle_timeline( req, tli_home );
		} else if( http_request_route( req, "public" ) ) {
			return handle_timeline( req, tli_public );
		}
	} else if( http_request_route( req, "apps/" ) ) {

		if( http_request_route( req, "verify_credentials" ) ) {
			http_request_send_headers( req, 200, "application/json", true );
			FILE* f = http_request_get_response_body( req );
			#include "src/view/api/verify_credentials.json.inc"
			return true;
		}

	} else if( http_request_route( req, "accounts/" ) ) {
		return route_api_account( req );
	}

	return false;
}

