#include <stdio.h>
#include <string.h>
#include <stddef.h>
#include <stdbool.h>
#include <unistd.h>
#include <signal.h>
#include <sys/prctl.h>

#include "app_args.h"
#include "http/server/server.h"
#include "http/server/request.h"

#include "controller/main.h"
#include "controller/inbox.h"
#include "controller/outbox.h"
#include "controller/test.h"
#include "controller/indexer.h"
#include "controller/cli.h"

#include <curl/curl.h>

bool terminate = false;

void handle_ctrl_c(int)
{
	terminate = true;
}

void handle_request( struct http_request* req, void* )
{
	printf( "Handling request from %s\n", http_request_get_remote_host_address( req ) );

	if( !route_request( req ) ) {
		FILE* f = http_request_get_response_body( req );
		http_request_send_headers( req, 404, "text/html", true );
		#include "view/404.html.inc"
	}

	fflush(stdout);
}

bool run_webserver( struct app_args* args )
{
	struct http_server* srv = http_server_new( args, handle_request, NULL );
	if( !srv ) {
		printf( "Error setting up server\n" );
		app_args_release(args);
		return false;
	}

	http_server_set_debug( srv, true );

	while(!terminate) {
		http_server_process( srv );
		usleep(25000);
	}

	http_server_release( srv );
	return true;
}

bool run_everything( struct app_args* args )
{
	// Process inbox
	//*
	int inbox_handler_pid = -1;
	if( !( inbox_handler_pid = fork() ) ) {
		prctl(PR_SET_PDEATHSIG, SIGHUP);
		process_inbox();
	}
	//*/

	//*
	// Process outbox
	int outbox_handler_pid = -1;
	if( !( outbox_handler_pid = fork() ) ) {
		prctl(PR_SET_PDEATHSIG, SIGHUP);
		process_outbox();
	}
	//*/

	int code = 0;
	if( !run_webserver(args) ) { return false; }

	return true;
}

void develop();

int main( int argc, char* argv[] )
{
	curl_global_init(CURL_GLOBAL_DEFAULT);

	signal(SIGINT, handle_ctrl_c);
	signal(SIGPIPE, SIG_IGN);

	struct app_args* args = app_args_new( argc, argv );
	if( !args ) {
		printf( "Error processing argument\n" );
		return 1;
	}

	int section = -1;

	if( ( argc > 1 ) && ( 0 != strncmp(argv[1],"--",2) ) ) {
		handle_command( argv, argc );
		goto exit;
	}

	for( int i = 1; i < argc; ++i ) {
		const char* arg = argv[i];

		if( sscanf(arg,"--section=%d",&section) ) {
		} else if( 0 == strcmp(argv[i],"--test") ) {
			section = 3;
		} else if( 0 == strcmp(argv[i],"--webserver") ) {
			section = 0;
		} else if( 0 == strcmp(argv[i],"--inbox") ) {
			section = 1;
		} else if( 0 == strcmp(argv[i],"--outbox") ) {
			section = 2;
		} else if( 0 == strcmp(argv[i],"--reindex") ) {
			section = 3;
		} else if( 0 == strcmp(argv[i],"--devel") ) {
			section = 100;
		} else {
			printf( "Unknown argument: %s\n", argv[i] );
			return 1;
		}
	}

	int code = 0;
	switch(section) {
		case 0:	code = !run_webserver(args); break;
		case 1:	process_inbox(); break;
		case 2:	process_outbox(); break;
		case 3:	built_in_test(); break;
		case 4:	reindex(); break;

		case 100:	develop(); break;
		default:	code = !run_everything(args); break;
	}

exit:
	app_args_release(args);

	return code;
}

