#include "../activity.h"

#include "json/layout.h"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stddef.h>

extern struct json_field_type ap_signature_composite_type;
extern struct json_field_type ap_activity_source_type;

// https://www.w3.org/TR/activitystreams-vocabulary/#h-activity-types
struct json_enum ap_activity_type_enum[] = {
	{ "Undo", apat_undo },
	{ "Follow", apat_follow },
	{ "Delete", apat_delete },
	{ "Accept", apat_accept },
	{ "Create", apat_create },
	{ "TentativeAccept", apat_tentative_accept },
	{ "Add", apat_add },
	{ "Arrive", apat_arrive },
	{ "Ignore", apat_ignore },
	{ "Join", apat_join },
	{ "Leave", apat_leave },
	{ "Like", apat_like },
	{ "Offer", apat_offer },
	{ "Invite", apat_invite },
	{ "Reject", apat_reject },
	{ "TentativeReject", apat_tentative_reject },
	{ "Remove", apat_remove },
	{ "Update", apat_update },
	{ "View", apat_view },
	{ "Listen", apat_listen },
	{ "Read", apat_read },
	{ "Move", apat_move },
	{ "Travel", apat_travel },
	{ "Announce", apat_announce },
	{ "Block", apat_block },
	{ "Flag", apat_flag },
	{ "Dislike", apat_dislike },
	{ "Question", apat_question },
	{ "EmojiReact", apat_emoji_react },
	{ "Note", apat_note },
	{ "Person", apat_person },
	{ NULL, 0 },
};

struct json_tagged_union_typelist activity_ref_types[] = {
	{
		.tag_value = apaot_ref,
		.offset = offsetof( struct ap_activity, object.ref ),
		.type = &json_field_string
	},
	{
		.tag_value = apaot_activity,
		.offset = offsetof( struct ap_activity, object.ptr ),
		.type = &ap_activity_type
	},
	JSON_TAGGED_UNION_TYPELIST_END,
};
struct json_tagged_union_typelist signature_types[] = {
	{
		.tag_value = 1,
		.offset = offsetof( struct ap_activity, signature ),
		.type = &ap_signature_composite_type,
	},
	JSON_TAGGED_UNION_TYPELIST_END,
};

#define OBJ_TYPE struct ap_activity
struct json_object_field ap_activity_layout[] = {
	{
		.key = "@context",
		.offset = offsetof( OBJ_TYPE, ap_context ),
		.required = false,
		.type = &ap_activity_context_type
	},

	JSON_FIELD_STRING(id,true),
	JSON_FIELD_STRING(actor,false),
	JSON_FIELD_STRING(state,false),

	{
		.key = "content",
		.offset = offsetof( struct ap_activity, content ),
		.required = false,
		.type = &ap_activity_source_type
	},
	JSON_FIELD_STRING(conversation,false),
	JSON_FIELD_DATETIME(published,false),
	{
		.key = "sensitive",
		.offset = offsetof( struct ap_activity, sensitive ),
		.required = false,
		.type = &json_field_bool_or_null
	},
	{
		.key = "source",
		.offset = offsetof( struct ap_activity, source ),
		.required = false,
		.type = &ap_activity_source_type
	},
	JSON_FIELD_STRING(summary,false),
	{
		.key = "inReplyTo",
		.offset = offsetof( OBJ_TYPE, in_reply_to ),
		.required = false,
		.type = &json_field_string
	},

	{
		.key = "tag",
		.offset = offsetof( struct ap_activity, tags ),
		.required = false,
		.type = &json_field_array_of,
		.array_item_type = &ap_activity_tag_type
	},
	{
		.key = "alsoKnownAs",
		.offset = offsetof( struct ap_activity, also_known_as ),
		.required = false,
		.type = &json_field_array_of,
		.array_item_type = &json_field_string,
	},

	JSON_FIELD_STRING(context,false),
	{
		.key = "attributedTo",
		.required = false,
		.offset = offsetof(OBJ_TYPE,attributed_to),
		.type = &json_field_string,
	},
	JSON_FIELD_STRING( target, false ),

	{
		.key = "directMessage",
		.offset = offsetof( struct ap_activity, direct_message ),
		.required = false,
		.type = &json_field_bool
	},

	{
		.key = "attachment",
		.offset = offsetof( struct ap_activity, attachments ),
		.required = false,
		.type = &json_field_array_of,
		.array_item_type = &ap_attachement_type
	},

	JSON_FIELD_ARRAY_OF_STRINGS(to,false),
	JSON_FIELD_ARRAY_OF_STRINGS(cc,false),
	JSON_FIELD_ARRAY_OF_STRINGS(bcc,false),

	{
		.key = "object",
		.offset = offsetof( struct ap_activity, object.tag ),
		.required = false,
		.type = &json_field_tagged_union,
		.tagged_item_types = activity_ref_types
	},

	{
		.key = "signature",
		.offset = offsetof( struct ap_activity, has_signature ),
		.required = false,
		.type = &json_field_tagged_union,
		.tagged_item_types = signature_types
	},

	JSON_FIELD_ENUM( type, ap_activity_type_enum, true ),

	JSON_FIELD_END,
};
#undef OBJ_TYPE

JSON_FIELD_TYPE_OBJECT_LAYOUT_WITH_DEFAULTS( ap_activity );

struct ap_activity* ap_activity_from_FILE( FILE* f )
{
	struct ap_activity* act = malloc(sizeof(struct ap_activity));
	memset(act,0,sizeof(*act));

	if( !json_read_object_layout_from_FILE( f, ap_activity_layout, act ) ) {
		ap_activity_free(act);
		return NULL;
	}

	if( !act->actor ) {
		if( !act->attributed_to ) {
			ap_activity_free(act);
			return NULL;
		}
		act->actor = strdup(act->attributed_to);
	}

	return act;
}

void ap_activity_write_to_FILE( struct ap_activity* act, FILE* f )
{
	struct json_writer jw = {
		.f = f,
		.indentation = "\t",
		.indent = 0,
	};
	json_write_pretty_object_layout( &jw, ap_activity_layout, act );
}
void ap_activity_save( struct ap_activity* act )
{
	char filename[512];
	snprintf( filename, sizeof(filename), "data/activities/%d.json", act->local_id );

	json_write_object_layout_to_file( filename, "\t", ap_activity_layout, act );
}
struct ap_activity* ap_activity_from_local_id( int id )
{
	struct ap_activity* act = ap_activity_new();

	char filename[512];
	snprintf( filename, sizeof(filename), "data/activities/%d.json", id );

	if( !json_read_object_layout_from_file( filename, ap_activity_layout, act ) ) {
		ap_activity_free(act);
		return NULL;
	}

	return act;
}

